@@ -72,9 +72,9 @@ struct RBPlayerPrivate
 
 	guint error_signal_id;
 
+	GstDParamManager *volume_dpmanager;
 	GstDParam *volume_dparam;
 	float cur_volume;
-	gboolean mute;
 
 	GTimer *timer;
 	long timer_add;
@@ -88,6 +88,7 @@ typedef enum
 	INFO,
 	BUFFERING_BEGIN,
 	BUFFERING_END,
+	BUFFERING_PROGRESS,
 	ERROR,
 	TICK,
 	LAST_SIGNAL
@@ -106,6 +107,7 @@ static guint rb_player_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
 static gboolean rb_player_sync_pipeline (RBPlayer *mp, gboolean iradio_mode, GError **error);
+static void rb_player_gst_free_pipeline (RBPlayer *player);
 
 GType
 rb_player_get_type (void)
@@ -179,6 +181,16 @@ rb_player_class_init (RBPlayerClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	rb_player_signals[BUFFERING_PROGRESS] =
+		g_signal_new ("buffering_progress",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerClass, buffering_progress),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__INT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_INT);
 	rb_player_signals[ERROR] =
 		g_signal_new ("error",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -243,7 +255,7 @@ rb_player_finalize (GObject *object)
 		gst_element_set_state (mp->priv->pipeline,
 				       GST_STATE_NULL);
 		
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+		rb_player_gst_free_pipeline (mp);
 	}
 
 	if (mp->priv->timer)
@@ -256,6 +268,21 @@ rb_player_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+rb_player_gst_free_pipeline (RBPlayer *player)
+{
+	if (player->priv->pipeline == NULL)
+		return;
+	
+	if (player->priv->volume_dpmanager) {
+		gst_dpman_detach_dparam (player->priv->volume_dpmanager, "volume");
+		gst_object_unref (GST_OBJECT (player->priv->volume_dparam));
+		player->priv->volume_dpmanager = NULL;
+	}
+	gst_object_unref (GST_OBJECT (player->priv->pipeline));
+	player->priv->pipeline = NULL;
+}
+
 static gboolean
 eos_signal_idle (RBPlayer *mp)
 {
@@ -428,16 +455,16 @@ rb_player_construct (RBPlayer *mp,
 	 *  { src ! spider ! volume ! sink }
 	 */
 	MAKE_ELEMENT_OR_LOSE(thread, pipeline);
-	g_signal_connect (G_OBJECT (mp->priv->pipeline),
-			  "deep_notify",
-			  G_CALLBACK (deep_notify_cb),
-			  mp);
+	g_signal_connect_object (G_OBJECT (mp->priv->pipeline),
+				 "deep_notify",
+				 G_CALLBACK (deep_notify_cb),
+				 mp, 0);
 
 	mp->priv->error_signal_id =
-		g_signal_connect (G_OBJECT (mp->priv->pipeline),
-				  "error",
-				  G_CALLBACK (error_cb),
-				  mp);
+		g_signal_connect_object (G_OBJECT (mp->priv->pipeline),
+					 "error",
+					 G_CALLBACK (error_cb),
+					 mp, 0);
 
 	/* Construct the two threads */
 	if (iradio_mode) {
@@ -484,8 +511,8 @@ rb_player_construct (RBPlayer *mp,
 		if (mp->priv->queue == NULL)
 			goto missing_element;
 		g_object_set (G_OBJECT (mp->priv->queue), "max-size-bytes", 64 * 1024, NULL);
-		g_signal_connect (G_OBJECT (mp->priv->queue), "overrun",
-				  G_CALLBACK (queue_full_cb), mp);
+		g_signal_connect_object (G_OBJECT (mp->priv->queue), "overrun",
+					 G_CALLBACK (queue_full_cb), mp, 0);
 		gst_bin_add (GST_BIN (mp->priv->srcthread), mp->priv->queue);
 	}
 
@@ -498,6 +525,7 @@ rb_player_construct (RBPlayer *mp,
 	MAKE_ELEMENT_OR_LOSE(volume, volume);
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->volume);
 	dpman = gst_dpman_get_manager (mp->priv->volume);
+	mp->priv->volume_dpmanager = dpman;
 	gst_dpman_set_mode (dpman, "synchronous");
 	mp->priv->volume_dparam = gst_dpsmooth_new (G_TYPE_DOUBLE);
 	g_assert (mp->priv->volume_dparam != NULL);
@@ -528,8 +556,8 @@ rb_player_construct (RBPlayer *mp,
 	else
 		gst_element_link_many (mp->priv->src, mp->priv->typefind, mp->priv->decoder, NULL);
 
-	g_signal_connect (G_OBJECT (mp->priv->sink), "eos",
-			  G_CALLBACK (eos_cb), mp);
+	g_signal_connect_object (G_OBJECT (mp->priv->sink), "eos",
+				 G_CALLBACK (eos_cb), mp, 0);
 
 	if (mp->priv->cur_volume > 1.0)
 		mp->priv->cur_volume = 1.0;
@@ -538,9 +566,6 @@ rb_player_construct (RBPlayer *mp,
 	g_object_set (G_OBJECT (mp->priv->volume_dparam),
 		      "value_double", mp->priv->cur_volume,
 		      NULL);
-	g_object_set (G_OBJECT (mp->priv->volume),
-		      "mute", mp->priv->mute,
-		      NULL);
 
 	if (mp->priv->timer)
 		g_timer_destroy (mp->priv->timer);
@@ -559,8 +584,7 @@ missing_element:
 			     RB_PLAYER_ERROR_GENERAL,
 			     err);
 		g_free (err);
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-		mp->priv->pipeline = NULL;
+		rb_player_gst_free_pipeline (mp);
 	}
 }
 
@@ -598,6 +622,8 @@ rb_player_error_quark (void)
 static gboolean
 rb_player_sync_pipeline (RBPlayer *mp, gboolean iradio_mode, GError **error)
 {
+	static gint may_pause = -1;
+  
 	rb_debug ("syncing pipeline");
 	if (mp->priv->playing) {
 		if (iradio_mode) {
@@ -625,6 +651,21 @@ rb_player_sync_pipeline (RBPlayer *mp, gboolean iradio_mode, GError **error)
 		}
 		g_timer_start (mp->priv->timer);
 	} else {
+		/* get correct gst version */
+		if (may_pause == -1) {
+			guint major, minor, micro;
+
+			rb_debug ("decoding gst version to check if we may free the audio sink");
+			gst_version (&major, &minor, &micro);
+			/* this makes sure someone removes this later on */
+			g_assert (major == 0);
+			g_assert (minor == 8);
+			if (micro > 1)
+				may_pause = 1;
+			else
+				may_pause = 0;
+		}
+		
 		rb_debug ("PAUSING pipeline");
 		if (gst_element_set_state (mp->priv->pipeline,
 					   GST_STATE_PAUSED) != GST_STATE_SUCCESS) {
@@ -635,16 +676,16 @@ rb_player_sync_pipeline (RBPlayer *mp, gboolean iradio_mode, GError **error)
 			return FALSE;
 		}
 			
-#if 0
-		rb_debug ("setting sink to NULL");
-		if (gst_element_set_state (mp->priv->sink, GST_STATE_NULL) != GST_STATE_SUCCESS) {
-			g_set_error (error,
-				     RB_PLAYER_ERROR,
-				     RB_PLAYER_ERROR_GENERAL,
-				     _("Could not close output sink"));
-			return FALSE;
+		if (may_pause == 1) {
+			rb_debug ("setting sink to NULL");
+			if (gst_element_set_state (mp->priv->sink, GST_STATE_NULL) != GST_STATE_SUCCESS) {
+				g_set_error (error,
+					     RB_PLAYER_ERROR,
+					     RB_PLAYER_ERROR_GENERAL,
+					     _("Could not close output sink"));
+				return FALSE;
+			}
 		}
-#endif
 	}
 	return TRUE;
 }
@@ -662,8 +703,7 @@ rb_player_open (RBPlayer *mp,
 	if (mp->priv->pipeline) {
 		gst_element_set_state (mp->priv->pipeline,
 				       GST_STATE_NULL);
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-		mp->priv->pipeline = NULL;
+		rb_player_gst_free_pipeline (mp);
 	}
 
 	g_free (mp->priv->uri);
@@ -790,8 +830,7 @@ rb_player_pause (RBPlayer *mp)
 	g_timer_stop (mp->priv->timer);
 	g_timer_reset (mp->priv->timer);
 
-	gst_element_set_state (mp->priv->pipeline,
-			       GST_STATE_PAUSED);
+	rb_player_sync_pipeline (mp, mp->priv->iradio_mode, NULL);
 }
 
 gboolean
@@ -870,30 +909,6 @@ rb_player_get_volume (RBPlayer *mp)
 	return mp->priv->cur_volume;
 }
 
-void
-rb_player_set_mute (RBPlayer *mp,
-			      gboolean mute)
-{
-	g_return_if_fail (RB_IS_PLAYER (mp));
-
-	if (mp->priv->pipeline != NULL)
-		g_object_set (G_OBJECT (mp->priv->volume),
-			      "mute",
-			      mute,
-			      NULL);
-	else
-
-	mp->priv->mute = mute;
-}
-
-gboolean
-rb_player_get_mute (RBPlayer *mp)
-{
-	g_return_val_if_fail (RB_IS_PLAYER (mp), FALSE);
-
-	return mp->priv->mute;
-}
-
 gboolean
 rb_player_seekable (RBPlayer *mp)
 {
